/*-----------------------------------------------------------------------------
 *      avrgcc_macros.h
 *
 *  Commonly used avr-gcc macro
 *-----------------------------------------------------------------------------
 *  Copyright (c) 2005-2011 Oleksandr Redchuk aka ReAl http://real.kiev.ua
 *  Public domain, AS IS.
 *-----------------------------------------------------------------------------
 */

#ifndef AVRGCC_MACROS_H
#define AVRGCC_MACROS_H

#ifndef NOINLINE
#define NOINLINE __attribute__((__noinline__))
#endif

#ifndef FORCE_INLINE
#define FORCE_INLINE __attribute__((__always_inline__))
#endif

#ifndef INLINE
#define INLINE  FORCE_INLINE
#endif

#ifndef NORETURN
#define NORETURN __attribute__((__noreturn__))
#endif

#ifndef PACKED
#define PACKED __attribute__((__packed__))
#endif

#ifndef OS_MAIN
#define OS_MAIN __attribute__((__OS_main__))
#endif

#ifndef OS_TASK
#define OS_TASK __attribute__((__OS_task__))
#endif


/*------------------------------------------------------------------------------
 *      INIT(number)
 *  Макрос для задання коду в секціях .init#number
 *  Для сумісності з попередніми версіями та для зручності аналізу згенерованого
 *  компілятором коду залишено варіант з двома аргументами
 *    INIT_CODE(number,name)
 *  name - ім'я псевдо-функції, можна використовувати
 *  як коментар. В межах файлу імена мають бути унікальними.
 *
 *  Наприклад:
 *  INIT(1)
 *  {
 *    MCUCR = (1 << SRE);
 *    XMCRA = (4 << SRL0) | (0 << SRW00);
 *    XMCRB = (1 << XMBK);
 *  }
 *  або
 *  INIT_CODE(1,setup_extram)
 *  {
 *    MCUCR = (1 << SRE);
 *    XMCRA = (4 << SRL0) | (0 << SRW00);
 *    XMCRB = (1 << XMBK);
 *  }
 *
 *  розподіл секцій для avr-gcc:
 *  .init0  Start here after reset.
 *  .init1
 *  .init2  Clear __zero_reg__, set up stack pointer.
 *  .init3
 *  .init4  Initialize data and BSS.
 *  .init5
 *  .init6  C++ constructors.
 *  .init7
 *  .init8
 *  .init9  Call main().
 *
 */

#define INIT_CODE(_number_,_name_) \
  __attribute__ ((section(".init" #_number_), naked, used)) \
  static void init_##_number_##_##_name_ (void)

#define INIT_CODE_(n,l) INIT_CODE(n,l)

#define INIT(_number_) INIT_CODE_(_number_,__LINE__)

#endif  /* AVRGCC_MACROS_H */

