/*-----------------------------------------------------------------------------
 *      .init sections usage demo (AVR/avr-gcc)
 *-----------------------------------------------------------------------------
 *  uart.c -- Minimalistic UART support
 *-----------------------------------------------------------------------------
 *  Copyright (c) 2011 Oleksandr Redchuk aka ReAl http://real.kiev.ua
 *  Public domain, AS IS.
 *-----------------------------------------------------------------------------
 */

#include "uart.h"
#include "avrgcc_macros.h"
#include "pin_macros.h"

#define UART0_BAUD 115200UL

#define BAUD_DIVIDER(baud,u2x) ( (F_CPU + ((u2x)?4:8) * (baud)) / (((u2x)?8:16)*(baud)) -1 )

#define TXD0 D,1,H

INIT(5)
{
	UCSR0A = 1 << U2X0;
	UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);
	UBRR0L = (uint8_t)  BAUD_DIVIDER(UART0_BAUD,1);
	UBRR0H = (uint8_t) (BAUD_DIVIDER(UART0_BAUD,1) >> 8);
	UCSR0B = (1<<RXEN0) | (1<<TXEN0);
	DRIVER(TXD0,OUT);
	uart_putstr_P(PSTR("\nUART module is initialised in .init5 section.\n\n"));
}

void uart_putchar(char ch)
{
    if (ch == '\n') uart_putchar('\r');

    while (!(UCSR0A & (1<<UDRE0))) {}
    UDR0 = ch;
}

void uart_putstr_P(const prog_char *str)
{
    char ch;
    while ((ch=pgm_read_byte(str++)) != 0) uart_putchar(ch);
}

